# *****************************************************************************************
#
# Script Name: copy_meta_temps.py
# Created by: Heather Stanton
# Created on: Mar 1, 2006
#
# Scipt Outline: Script copies metadata from user specified location based on the feature
#                classes contained in the geodatabases in the input geodatabase location.
#                If the metadata file exists in the output location, it will not be copied.
#                Script renames the template metadata file to be consistent with files
#                contained within each of the input geodatabases.
#
# Input Requirements: User specified location of input geodatabases, location of metadata
#                     templates and the park code for which templates will be copied 
#
# Script Requirements: 1.) All user input is required.
#                      2.) All files must adhere to GRE file naming conventions.
#                       
# *****************************************************************************************
import sys, string, os, arcgisscripting, win32file, time
from MetaReplace import *
from GetReplaceTerms import *

# Create the Geoprocessor object
GP = arcgisscripting.create()

Input_GDBs = sys.argv[1]
Template_Location = sys.argv[2]
SR_Geo = sys.argv[3]
Map_Info_Table = sys.argv[4]
Proc_Contact = sys.argv[5]
Meta_Contact = sys.argv[6]
Park_Table = sys.argv[7]
Out_Location = sys.argv[8]

# Fix names with spaces in them so that they don't have quotes around them
Proc_Contact = Proc_Contact.replace("'", "")
Meta_Contact = Meta_Contact.replace("'", "")

Park_GDB = ""

today = time.strftime("%Y/%m/%d") #Year,Month,Day (capital "Y" for 4-digit year), thanks to Jacob Morgan
Date = today.replace("/", "")

GDB_List = Input_GDBs.split(";") # Create list of input GDBs for processing

for GDB in GDB_List: # For each geodatabase in the list...
    GP.Workspace = GDB
      
    fds = GP.ListDatasets("*", "featuredataset")
    GMAP_Code = fds.Next() # Get the GMAP code from the feature dataset
    States = GetMapInfo(GP, GMAP_Code, "States", Map_Info_Table)
    # Split the Park_Code and Park_Name into an array
    Park_Codes = GetMapInfo(GP, GMAP_Code, "ParkCode", Map_Info_Table)
    PCs = Park_Codes.split(";")
    Park_Names = GetParkNames(GP, PCs, Map_Info_Table)
    PNs = Park_Names.split(";")
    Map_Title = GetMapInfo(GP, GMAP_Code, "MapTitle", Map_Info_Table)

    STs = States
    while STs.count(";") > 0:
        if STs.count(";") == 1:
            STs = STs.replace(";", " and ")
        else:
            STs = STs.replace(";", ", ", 1)
            
    LW_Title = Map_Title + ", " + STs
    i = LW_Title.find("Map ")
    DME_Title = LW_Title[(i + 4):]
    
    if PCs[0].lower() == GMAP_Code.lower():
        Park_LW_Title = LW_Title
        Park_DME_Title = DME_Title
        Park_GDB = GDB
        
    Credit = GetMapInfo(GP, GMAP_Code, "Credit", Map_Info_Table)
    while GMAP_Code: # For each GMAP code feature dataset...
        Input_GDB_Location = GP.Describe(GDB).CatalogPath
        Input_GDB_Location = Input_GDB_Location.replace("\\" + GP.Describe(GDB).Name, "")
        #Input_GDB_Location = GDB.replace("\\" + GMAP_Code + "_geology.mdb", "")
        GP.AddMessage("Copying templates for " + GMAP_Code + " feature dataset in " + GDB + "...\n")
        GP.Workspace = GDB + "\\" + GMAP_Code
        fclist = ""

        fcs = GP.ListFeatureClasses() # Get the list of feature classes in the feature dataset
        fc = fcs.Next()
        while fc: # For each feature class

            meta_file = "codefc.txt"

            out_file = fc + ".txt"

            #Get extracted terms for search and replace tool
            desc = GP.Describe(fc)
            SR = desc.SpatialReference
            FC_Alias = desc.AliasName

            # Get spatial reference terms
            SRName = SR.Name
            SRCM = str(SR.CentralMeridianInDegrees)
            UTM_Zone = SRName.replace("NAD_1983_UTM_Zone_", "")
            UTM_Zone = UTM_Zone.replace("N","")

            # Get geometry terms
            Geometry = desc.ShapeType
            if Geometry == "Polyline":
                SDTS_Type = "String"
            elif Geometry == "Polygon":
                SDTS_Type = "G-polygon"
            elif Geometry == "Point":
                SDTS_Type = "Entity point"
            Feat_Count = str(GP.GetCount(fc)) # Get the number of features
            
            # If the metadata template exists and the output metadata file does not exist, replace
            # park-wide terms, contact terms, and source terms after getting geographic extent
            if GP.Exists (Template_Location + "\\" + meta_file):
                if not GP.Exists(Input_GDB_Location + "\\" + out_file):

                    # Get bounding coordinates
                    GP.RefreshCatalog(Input_GDB_Location)
                    Geo_Extent = GetGeoExtent(GP, SR_Geo, fc, Input_GDB_Location)

                    if fc.endswith("glg"):
                        Map_Extent = Geo_Extent
                    elif not GP.Exists(GMAP_Code + "glg") and fc.endswith("sur"):
                        Map_Extent = Geo_Extent
                    elif not GP.Exists(GMAP_Code + "glg") and not GP.Exists(GMAP_Code + "sur") and fc.endswith("lan"):
                        Map_Extent = Geo_Extent
                    
                    SearchAndReplacePark(GP, GDB, Template_Location + "\\" + meta_file, Input_GDB_Location + "\\" + out_file,
                                         GMAP_Code, Park_Codes, Park_Names, States, UTM_Zone, SRName, SRCM, Feat_Count, Geometry, SDTS_Type,
                                         Geo_Extent, FC_Alias, fc, Date)

                    SearchAndReplaceContact(GP, Template_Location, Input_GDB_Location + "\\" + out_file, Proc_Contact, Meta_Contact)
                    
##                    SearchAndReplaceSource(GP, Input_GDB_Location, Template_Location, Input_GDB_Location + "\\" + out_file,
##                                           fc, States, Park_Table, PCs[0])

                    SearchAndReplaceQuad(GP, Input_GDB_Location + "\\" + out_file, Template_Location, Credit, LW_Title, DME_Title)


                    GP.AddMessage("**Search and replace successfully completed for " + out_file + "**")  
                elif fc.endswith("glg") or (not GP.Exists(GMAP_Code + "glg") and fc.endswith("sur")) or (
                    not GP.Exists(GMAP_Code + "glg") and not GP.Exists(GMAP_Code + "sur") and fc.endswith("lan")):
                    # Get bounding coordinates
                    Map_Extent = GetGeoExtent(GP, SR_Geo, fc, Input_GDB_Location)
                    GP.AddMessage("\nMetadata file " + out_file + " already exists\n")
                else:
                    GP.AddMessage("\nMetadata file " + out_file + " already exists\n")
            else:
                GP.AddWarning("\n***Metadata template for " + fc + " does not exist***\n")

            if fclist == "":
                fclist = fc
            else:
                fclist = fclist + ";" + fc
                
            fc = fcs.Next()

        # Search and replace terms in the _metadata record for each GMAP code
        if not GP.Exists(Input_GDB_Location + "\\" + GMAP_Code.lower() + "_metadata.txt"):
            meta_file = "code_metadata.txt"

            out_file = GMAP_Code.lower() + "_metadata.txt"
            SearchAndReplacePark(GP, GDB, Template_Location + "\\" + meta_file, Input_GDB_Location + "\\" + out_file,
                                 GMAP_Code, Park_Codes, Park_Names, States, UTM_Zone, SRName, SRCM, "", "", "", Map_Extent, "", "", Date)
            SearchAndReplaceContact(GP, Template_Location, Input_GDB_Location + "\\" + out_file, Proc_Contact, Meta_Contact)
            SearchAndReplaceSource(GP, Input_GDB_Location, Template_Location, Input_GDB_Location + "\\" + out_file, fclist, States, Park_Table, PCs[0])
            SearchAndReplaceQuad(GP, Input_GDB_Location + "\\" + out_file, Template_Location, Credit, LW_Title, DME_Title)
            if Park_Codes.find(GMAP_Code) == -1:
                Map_Name = GetMapInfo(GP, GMAP_Code, "MapName", Map_Info_Table)
                AddPlaceKeyword(Input_GDB_Location + "\\" + out_file, Template_Location, Map_Name)
            GP.AddMessage("\nMap metadata for " + GMAP_Code + " has been completed")
        elif GP.Exists(Input_GDB_Location + "\\" + GMAP_Code.lower() + "_metadata.txt"):
            GP.AddMessage("Metadata file " + GMAP_Code.lower() + "_metadata.txt already exists")

 
        GP.RefreshCatalog(Input_GDB_Location)
            
        GMAP_Code = fds.Next()
        GP.AddMessage("\n")
        
if Park_Codes.find(";") <> -1:
    GMAP_Code = PCs[0].lower()
else:
    GMAP_Code = Park_Codes

if not Park_GDB == "" or (Park_GDB == "" and Out_Location <> ""):
    if not Park_GDB == "":
        Input_GDB_Location = GP.Describe(Park_GDB).CatalogPath
        Input_GDB_Location = Input_GDB_Location.replace("\\" + GP.Describe(Park_GDB).Name, "")
    else:
        Input_GDB_Location = Out_Location
        Park_LW_Title = "#LWTTL#"
        Park_DME_Title = "#DMETTL#"
        GP.AddWarning("*****Unable to replace Park Larger Work Title in readme file!!!*****\n\n")
    GP.AddMessage("The table and readme location is " + Input_GDB_Location)
    # Search and replace terms in the ancillary table, help and readme templates to input geodatabases location
    if not GP.Exists(Input_GDB_Location + "\\" + GMAP_Code.lower() + "unit.txt"):
        meta_file = "codefc.txt"
        out_file = GMAP_Code.lower() + "unit.txt"
                    
        SearchAndReplacePark(GP, GDB, Template_Location + "\\" + meta_file, Input_GDB_Location + "\\" + out_file,
                             GMAP_Code, Park_Codes, Park_Names, States, UTM_Zone, "", "", "", "", "",
                             Map_Extent, "Geologic Unit Information Table", PCs[0] + "unit", Date)

        SearchAndReplaceContact(GP, Template_Location, Input_GDB_Location + "\\" + out_file, Proc_Contact, Meta_Contact)

        SearchAndReplaceQuad(GP, Input_GDB_Location + "\\" + out_file, Template_Location, Credit, Park_LW_Title, Park_DME_Title)

        GP.AddMessage("\nMetadata file " + GMAP_Code.lower() + "unit.txt has been completed")
    else:
        GP.AddMessage("Metadata file " + Input_GDB_Location + "\\" + GMAP_Code.lower() + "unit.txt" + " already exists")
        
    if not GP.Exists(Input_GDB_Location + "\\" + GMAP_Code.lower() + "map.txt"):
        meta_file = "codefc.txt"
        out_file = GMAP_Code.lower() + "map.txt"
        SearchAndReplacePark(GP, GDB, Template_Location + "\\" + meta_file, Input_GDB_Location + "\\" + out_file,
                             GMAP_Code, Park_Codes, Park_Names, States, UTM_Zone, "", "", "", "", "",
                             Map_Extent, "Source Map Information Table", PCs[0] + "map", Date)

        SearchAndReplaceContact(GP, Template_Location, Input_GDB_Location + "\\" + out_file, Proc_Contact, Meta_Contact)

        SearchAndReplaceQuad(GP, Input_GDB_Location + "\\" + out_file, Template_Location, Credit, Park_LW_Title, Park_DME_Title)        
        GP.AddMessage("Metadata file " + GMAP_Code.lower() + "map.txt has been completed")
    else:
        GP.AddMessage("Metadata file " + Input_GDB_Location + "\\" + GMAP_Code.lower() + "map.txt" + " already exists")
        
##    if not GP.Exists(Input_GDB_Location + "\\readme.txt"):
##        meta_file = "readme.txt"
##        out_file = "readme.txt"
##        SearchAndReplacePark(GP, GDB, Template_Location + "\\" + meta_file, Input_GDB_Location + "\\" + out_file,
##                             GMAP_Code, Park_Codes, Park_Names, States, UTM_Zone, "", "", "", "", "", Map_Extent, "", "", Date)
##        SearchAndReplaceQuad(GP, Input_GDB_Location + "\\" + out_file, Template_Location, Credit, Park_LW_Title, Park_DME_Title)
##
##        GP.AddMessage("File readme.txt has been completed")
##    else:
##        GP.AddMessage("Metadata file " + Input_GDB_Location + "\\readme.txt" + " already exists")
##        
##    if not GP.Exists(Input_GDB_Location + "\\readme1.txt"):
##        meta_file = "readme1.txt"
##        out_file = "readme1.txt"
##        SearchAndReplacePark(GP, GDB, Template_Location + "\\" + meta_file, Input_GDB_Location + "\\" + out_file,
##                             GMAP_Code, Park_Codes, Park_Names, States, UTM_Zone, "", "", "", "", "", Map_Extent, "", "", Date)
##        SearchAndReplaceQuad(GP, Input_GDB_Location + "\\" + out_file, Template_Location, Credit, Park_LW_Title, Park_DME_Title)
##        
##        GP.AddMessage("File readme1.txt has been completed\n")
##    else:
##        GP.AddMessage("Metadata file " + Input_GDB_Location + "\\readme1.txt" + " already exists")
    GP.RefreshCatalog(Input_GDB_Location)
else:
    GP.AddWarning("Unable to create readme files or map and unit table metadata records.  Park geodatabase or output location not specified!!!")
